@@ -59,11 +59,11 @@ class Akismet_Admin {
 
 		add_filter( 'plugin_action_links', array( 'Akismet_Admin', 'plugin_action_links' ), 10, 2 );
 		add_filter( 'comment_row_actions', array( 'Akismet_Admin', 'comment_row_action' ), 10, 2 );
-		
+
 		add_filter( 'plugin_action_links_'.plugin_basename( plugin_dir_path( __FILE__ ) . 'akismet.php'), array( 'Akismet_Admin', 'admin_plugin_settings_link' ) );
-		
+
 		add_filter( 'wxr_export_skip_commentmeta', array( 'Akismet_Admin', 'exclude_commentmeta_from_export' ), 10, 3 );
-		
+
 		add_filter( 'all_plugins', array( 'Akismet_Admin', 'modify_plugin_description' ) );
 
 		// priority=1 because we need ours to run before core's comment anonymizer runs, and that's registered at priority=10
@@ -85,7 +85,7 @@ class Akismet_Admin {
 		if ( function_exists( 'wp_add_privacy_policy_content' ) ) {
 			wp_add_privacy_policy_content(
 				__( 'Akismet', 'akismet' ),
-				__( 'We collect information about visitors who comment on Sites that use our Akismet anti-spam service. The information we collect depends on how the User sets up Akismet for the Site, but typically includes the commenter\'s IP address, user agent, referrer, and Site URL (along with other information directly provided by the commenter such as their name, username, email address, and the comment itself).', 'akismet' )
+				__( 'We collect information about visitors who comment on Sites that use our Akismet Anti-spam service. The information we collect depends on how the User sets up Akismet for the Site, but typically includes the commenter\'s IP address, user agent, referrer, and Site URL (along with other information directly provided by the commenter such as their name, username, email address, and the comment itself).', 'akismet' )
 			);
 		}
 	}
@@ -102,21 +102,21 @@ class Akismet_Admin {
 		if ( !current_user_can( 'manage_options' ) )
 			return;
 	}
-	
-	public static function admin_plugin_settings_link( $links ) { 
+
+	public static function admin_plugin_settings_link( $links ) {
   		$settings_link = '<a href="'.esc_url( self::get_page_url() ).'">'.__('Settings', 'akismet').'</a>';
-  		array_unshift( $links, $settings_link ); 
-  		return $links; 
+		array_unshift( $links, $settings_link );
+		return $links;
 	}
 
 	public static function load_menu() {
 		if ( class_exists( 'Jetpack' ) ) {
-			$hook = add_submenu_page( 'jetpack', __( 'Akismet Anti-Spam' , 'akismet'), __( 'Akismet Anti-Spam' , 'akismet'), 'manage_options', 'akismet-key-config', array( 'Akismet_Admin', 'display_page' ) );
+			$hook = add_submenu_page( 'jetpack', __( 'Akismet Anti-spam', 'akismet' ), __( 'Akismet Anti-spam', 'akismet' ), 'manage_options', 'akismet-key-config', array( 'Akismet_Admin', 'display_page' ) );
 		}
 		else {
-			$hook = add_options_page( __('Akismet Anti-Spam', 'akismet'), __('Akismet Anti-Spam', 'akismet'), 'manage_options', 'akismet-key-config', array( 'Akismet_Admin', 'display_page' ) );
+			$hook = add_options_page( __( 'Akismet Anti-spam', 'akismet' ), __( 'Akismet Anti-spam', 'akismet' ), 'manage_options', 'akismet-key-config', array( 'Akismet_Admin', 'display_page' ) );
 		}
-		
+
 		if ( $hook ) {
 			add_action( "load-$hook", array( 'Akismet_Admin', 'admin_help' ) );
 		}
@@ -134,12 +134,23 @@ class Akismet_Admin {
 			'jetpack_page_akismet-key-config',
 			'plugins.php',
 		) ) ) ) {
-			wp_register_style( 'akismet.css', plugin_dir_url( __FILE__ ) . '_inc/akismet.css', array(), AKISMET_VERSION );
-			wp_enqueue_style( 'akismet.css');
+			$akismet_css_path = is_rtl() ? '_inc/rtl/akismet-rtl.css' : '_inc/akismet.css';
+			wp_register_style( 'akismet', plugin_dir_url( __FILE__ ) . $akismet_css_path, array(), self::get_asset_file_version( $akismet_css_path ) );
+			wp_enqueue_style( 'akismet' );
 
-			wp_register_script( 'akismet.js', plugin_dir_url( __FILE__ ) . '_inc/akismet.js', array('jquery'), AKISMET_VERSION );
+			wp_register_style( 'akismet-font-inter', plugin_dir_url( __FILE__ ) . '_inc/fonts/inter.css', array(), self::get_asset_file_version( '_inc/fonts/inter.css' ) );
+			wp_enqueue_style( 'akismet-font-inter' );
+
+			$akismet_admin_css_path = is_rtl() ? '_inc/rtl/akismet-admin-rtl.css' : '_inc/akismet-admin.css';
+			wp_register_style( 'akismet-admin', plugin_dir_url( __FILE__ ) . $akismet_admin_css_path, array(), self::get_asset_file_version( $akismet_admin_css_path ) );
+			wp_enqueue_style( 'akismet-admin' );
+
+			wp_register_script( 'akismet.js', plugin_dir_url( __FILE__ ) . '_inc/akismet.js', array( 'jquery' ), self::get_asset_file_version( '_inc/akismet.js' ) );
 			wp_enqueue_script( 'akismet.js' );
-		
+
+			wp_register_script( 'akismet-admin.js', plugin_dir_url( __FILE__ ) . '_inc/akismet-admin.js', array(), self::get_asset_file_version( '/_inc/akismet-admin.js' ) );
+			wp_enqueue_script( 'akismet-admin.js' );
+
 			$inline_js = array(
 				'comment_author_url_nonce' => wp_create_nonce( 'comment_author_url_nonce' ),
 				'strings' => array(
@@ -293,7 +304,7 @@ class Akismet_Admin {
 		if ( Akismet::predefined_api_key() ) {
 			return false; //shouldn't have option to save key if already defined
 		}
-		
+
 		$new_key = preg_replace( '/[^a-f0-9]/i', '', $_POST['key'] );
 		$old_key = Akismet::get_api_key();
 
@@ -315,11 +326,11 @@ class Akismet_Admin {
 
 		if ( $key_status == 'valid' ) {
 			$akismet_user = self::get_akismet_user( $api_key );
-			
-			if ( $akismet_user ) {				
+
+			if ( $akismet_user ) {
 				if ( in_array( $akismet_user->status, array( 'active', 'active-dunning', 'no-sub' ) ) )
 					update_option( 'wordpress_api_key', $api_key );
-				
+
 				if ( $akismet_user->status == 'active' )
 					self::$notices['status'] = 'new-key-valid';
 				elseif ( $akismet_user->status == 'notice' )
@@ -392,7 +403,7 @@ class Akismet_Admin {
 		$link = '';
 
 		$comments_count = wp_count_comments();
-		
+
 		echo '</div>';
 		echo '<div class="alignleft actions">';
 
@@ -433,7 +444,7 @@ class Akismet_Admin {
 		if ( ! ( isset( $_GET['recheckqueue'] ) || ( isset( $_REQUEST['action'] ) && 'akismet_recheck_queue' == $_REQUEST['action'] ) ) ) {
 			return;
 		}
-		
+
 		if ( ! wp_verify_nonce( $_POST['nonce'], 'akismet_check_for_spam' ) ) {
 			wp_send_json( array(
 				'error' => __( 'You don&#8217;t have permission to do that.', 'akismet' ),
@@ -454,10 +465,10 @@ class Akismet_Admin {
 			exit;
 		}
 	}
-	
+
 	public static function recheck_queue_portion( $start = 0, $limit = 100 ) {
 		global $wpdb;
-		
+
 		$paginate = '';
 
 		if ( $limit <= 0 ) {
@@ -471,7 +482,7 @@ class Akismet_Admin {
 		$moderation = $wpdb->get_col( $wpdb->prepare( "SELECT * FROM {$wpdb->comments} WHERE comment_approved = '0' LIMIT %d OFFSET %d", $limit, $start ) );
 
 		$result_counts = array(
-			'processed' => count( $moderation ),
+			'processed' => is_countable( $moderation ) ? count( $moderation ) : 0,
 			'spam' => 0,
 			'ham' => 0,
 			'error' => 0,
@@ -544,7 +555,7 @@ class Akismet_Admin {
 		}
 
 		// add a History item to the hover links, just after Edit
-		if ( $akismet_result ) {
+		if ( $akismet_result && is_array( $a ) ) {
 			$b = array();
 			foreach ( $a as $k => $item ) {
 				$b[ $k ] = $item;
@@ -563,15 +574,15 @@ class Akismet_Admin {
 			echo '<span class="akismet-status" commentid="'.$comment->comment_ID.'"><a href="comment.php?action=editcomment&amp;c='.$comment->comment_ID.'#akismet-status" title="' . esc_attr__( 'View comment history' , 'akismet') . '">'.esc_html( $desc ).'</a></span>';
 
 		$show_user_comments_option = get_option( 'akismet_show_user_comments_approved' );
-		
+
 		if ( $show_user_comments_option === false ) {
 			// Default to active if the user hasn't made a decision.
 			$show_user_comments_option = '1';
 		}
-		
+
 		$show_user_comments = apply_filters( 'akismet_show_user_comments_approved', $show_user_comments_option );
 		$show_user_comments = $show_user_comments === 'false' ? false : $show_user_comments; //option used to be saved as 'false' / 'true'
-		
+
 		if ( $show_user_comments ) {
 			$comment_count = Akismet::get_user_comments_approved( $comment->user_id, $comment->comment_author_email, $comment->comment_author, $comment->comment_author_url );
 			$comment_count = intval( $comment_count );
@@ -592,7 +603,7 @@ class Akismet_Admin {
 					// Old versions of Akismet stored the message as a literal string in the commentmeta.
 					// New versions don't do that for two reasons:
 					// 1) Save space.
-					// 2) The message can be translated into the current language of the blog, not stuck 
+					// 2) The message can be translated into the current language of the blog, not stuck
 					//    in the language of the blog when the comment was made.
 					$message = esc_html( $row['message'] );
 				} else if ( ! empty( $row['event'] ) ) {
@@ -679,7 +690,7 @@ class Akismet_Admin {
 					echo '<p>';
 
 					if ( isset( $row['time'] ) ) {
-						$time = gmdate( 'D d M Y @ h:i:s a', $row['time'] ) . ' GMT';
+						$time = gmdate( 'D d M Y @ h:i:s a', (int) $row['time'] ) . ' GMT';
 
 						/* translators: The placeholder is an amount of time, like "7 seconds" or "3 days" returned by the function human_time_diff(). */
 						$time_html = '<span style="color: #999;" alt="' . esc_attr( $time ) . '" title="' . esc_attr( $time ) . '">' . sprintf( esc_html__( '%s ago', 'akismet' ), human_time_diff( $row['time'] ) ) . '</span>';
@@ -738,16 +749,16 @@ class Akismet_Admin {
 	// Check connectivity between the WordPress blog and Akismet's servers.
 	// Returns an associative array of server IP addresses, where the key is the IP address, and value is true (available) or false (unable to connect).
 	public static function check_server_ip_connectivity() {
-		
+
 		$servers = $ips = array();
 
 		// Some web hosts may disable this function
-		if ( function_exists('gethostbynamel') ) {	
-			
+		if ( function_exists( 'gethostbynamel' ) ) {
+
 			$ips = gethostbynamel( 'rest.akismet.com' );
 			if ( $ips && is_array($ips) && count($ips) ) {
 				$api_key = Akismet::get_api_key();
-				
+
 				foreach ( $ips as $ip ) {
 					$response = Akismet::verify_key( $api_key, $ip );
 					// even if the key is invalid, at least we know we have connectivity
@@ -758,13 +769,13 @@ class Akismet_Admin {
 				}
 			}
 		}
-		
+
 		return $servers;
 	}
-	
+
 	// Simpler connectivity check
 	public static function check_server_connectivity($cache_timeout = 86400) {
-		
+
 		$debug = array();
 		$debug[ 'PHP_VERSION' ]         = PHP_VERSION;
 		$debug[ 'WORDPRESS_VERSION' ]   = $GLOBALS['wp_version'];
@@ -772,7 +783,7 @@ class Akismet_Admin {
 		$debug[ 'AKISMET__PLUGIN_DIR' ] = AKISMET__PLUGIN_DIR;
 		$debug[ 'SITE_URL' ]            = site_url();
 		$debug[ 'HOME_URL' ]            = home_url();
-		
+
 		$servers = get_option('akismet_available_servers');
 		if ( (time() - get_option('akismet_connectivity_time') < $cache_timeout) && $servers !== false ) {
 			$servers = self::check_server_ip_connectivity();
@@ -790,16 +801,16 @@ class Akismet_Admin {
 		$debug[ 'gethostbynamel' ]  = function_exists('gethostbynamel') ? 'exists' : 'not here';
 		$debug[ 'Servers' ]         = $servers;
 		$debug[ 'Test Connection' ] = $response;
-		
+
 		Akismet::log( $debug );
-		
+
 		if ( $response && 'connected' == wp_remote_retrieve_body( $response ) )
 			return true;
-		
+
 		return false;
 	}
 
-	// Check the server connectivity and store the available servers in an option. 
+	// Check the server connectivity and store the available servers in an option.
 	public static function get_server_connectivity($cache_timeout = 86400) {
 		return self::check_server_connectivity( $cache_timeout );
 	}
@@ -814,10 +825,10 @@ class Akismet_Admin {
 			// Exclude comments that are not pending. This would happen if someone manually approved or spammed a comment
 			// that was waiting to be checked. The akismet_error meta entry will eventually be removed by the cron recheck job.
 			'status' => 'hold',
-			
+
 			// This is the commentmeta that is saved when a comment couldn't be checked.
 			'meta_key' => 'akismet_error',
-			
+
 			// We only need to know whether at least one comment is waiting for a check.
 			'number' => 1,
 		) );
@@ -837,11 +848,18 @@ class Akismet_Admin {
 
 		return add_query_arg( $args, menu_page_url( 'akismet-key-config', false ) );
 	}
-	
+
 	public static function get_akismet_user( $api_key ) {
 		$akismet_user = false;
 
-		$subscription_verification = Akismet::http_post( Akismet::build_query( array( 'key' => $api_key, 'blog' => get_option( 'home' ) ) ), 'get-subscription' );
+		$request_args = array(
+			'key' => $api_key,
+			'blog' => get_option( 'home' ),
+		);
+
+		$request_args = apply_filters( 'akismet_request_args', $request_args, 'get-subscription' );
+
+		$subscription_verification = Akismet::http_post( Akismet::build_query( $request_args ), 'get-subscription' );
 
 		if ( ! empty( $subscription_verification[1] ) ) {
 			if ( 'invalid' !== $subscription_verification[1] ) {
@@ -851,12 +869,20 @@ class Akismet_Admin {
 
 		return $akismet_user;
 	}
-	
+
 	public static function get_stats( $api_key ) {
 		$stat_totals = array();
 
 		foreach( array( '6-months', 'all' ) as $interval ) {
-			$response = Akismet::http_post( Akismet::build_query( array( 'blog' => get_option( 'home' ), 'key' => $api_key, 'from' => $interval ) ), 'get-stats' );
+			$request_args = array(
+				'blog' => get_option( 'home' ),
+				'key' => $api_key,
+				'from' => $interval,
+			);
+
+			$request_args = apply_filters( 'akismet_request_args', $request_args, 'get-stats' );
+
+			$response = Akismet::http_post( Akismet::build_query( $request_args ), 'get-stats' );
 
 			if ( ! empty( $response[1] ) ) {
 				$stat_totals[$interval] = json_decode( $response[1] );
@@ -865,35 +891,42 @@ class Akismet_Admin {
 
 		return $stat_totals;
 	}
-	
+
 	public static function verify_wpcom_key( $api_key, $user_id, $extra = array() ) {
-		$akismet_account = Akismet::http_post( Akismet::build_query( array_merge( array(
-			'user_id'          => $user_id,
-			'api_key'          => $api_key,
-			'get_account_type' => 'true'
-		), $extra ) ), 'verify-wpcom-key' );
+		$request_args = array_merge(
+			array(
+				'user_id' => $user_id,
+				'api_key' => $api_key,
+				'get_account_type' => 'true',
+			),
+			$extra
+		);
+
+		$request_args = apply_filters( 'akismet_request_args', $request_args, 'verify-wpcom-key' );
+
+		$akismet_account = Akismet::http_post( Akismet::build_query( $request_args ), 'verify-wpcom-key' );
 
 		if ( ! empty( $akismet_account[1] ) )
 			$akismet_account = json_decode( $akismet_account[1] );
 
 		Akismet::log( compact( 'akismet_account' ) );
-		
+
 		return $akismet_account;
 	}
-	
+
 	public static function connect_jetpack_user() {
-	
+
 		if ( $jetpack_user = self::get_jetpack_user() ) {
 			if ( isset( $jetpack_user['user_id'] ) && isset(  $jetpack_user['api_key'] ) ) {
 				$akismet_user = self::verify_wpcom_key( $jetpack_user['api_key'], $jetpack_user['user_id'], array( 'action' => 'connect_jetpack_user' ) );
-							
+
 				if ( is_object( $akismet_user ) ) {
 					self::save_key( $akismet_user->api_key );
 					return in_array( $akismet_user->status, array( 'active', 'active-dunning', 'no-sub' ) );
 				}
 			}
 		}
-		
+
 		return false;
 	}
 
@@ -964,18 +997,18 @@ class Akismet_Admin {
 			self::display_configuration_page();
 			return;
 		}
-		
+
 		//the user can choose to auto connect their API key by clicking a button on the akismet done page
 		//if jetpack, get verified api key by using connected wpcom user id
-		//if no jetpack, get verified api key by using an akismet token	
-		
+		//if no jetpack, get verified api key by using an akismet token
+
 		$akismet_user = false;
-		
+
 		if ( isset( $_GET['token'] ) && preg_match('/^(\d+)-[0-9a-f]{20}$/', $_GET['token'] ) )
 			$akismet_user = self::verify_wpcom_key( '', '', array( 'token' => $_GET['token'] ) );
 		elseif ( $jetpack_user = self::get_jetpack_user() )
 			$akismet_user = self::verify_wpcom_key( $jetpack_user['api_key'], $jetpack_user['user_id'] );
-			
+
 		if ( isset( $_GET['action'] ) ) {
 			if ( $_GET['action'] == 'save-key' ) {
 				if ( is_object( $akismet_user ) ) {
@@ -1009,7 +1042,7 @@ class Akismet_Admin {
 	public static function display_configuration_page() {
 		$api_key      = Akismet::get_api_key();
 		$akismet_user = self::get_akismet_user( $api_key );
-		
+
 		if ( ! $akismet_user ) {
 			// This could happen if the user's key became invalid after it was previously valid and successfully set up.
 			self::$notices['status'] = 'existing-key-invalid';
@@ -1024,7 +1057,7 @@ class Akismet_Admin {
 		if ( get_option( 'akismet_strictness' ) === false ) {
 			add_option( 'akismet_strictness', ( get_option( 'akismet_discard_month' ) === 'false' ? '0' : '1' ) );
 		}
-		
+
 		// Sync the local "Total spam blocked" count with the authoritative count from the server.
 		if ( isset( $stat_totals['all'], $stat_totals['all']->spam ) ) {
 			update_option( 'akismet_spam_count', $stat_totals['all']->spam );
@@ -1050,15 +1083,11 @@ class Akismet_Admin {
 					elseif ( $total_in_minutes >= 30 )
 						$time_saved = $cleaning_up . ' ' . sprintf( _n( 'Akismet has saved you %d minute!', 'Akismet has saved you %d minutes!', $total_in_minutes, 'akismet' ), $total_in_minutes );
 				}
-				
+
 				$notices[] =  array( 'type' => 'active-notice', 'time_saved' => $time_saved );
 			}
-			
-			if ( !empty( $akismet_user->limit_reached ) && in_array( $akismet_user->limit_reached, array( 'yellow', 'red' ) ) ) {
-				$notices[] = array( 'type' => 'limit-reached', 'level' => $akismet_user->limit_reached );
-			}
 		}
-		
+
 		if ( !isset( self::$notices['status'] ) && in_array( $akismet_user->status, array( 'cancelled', 'suspended', 'missing', 'no-sub' ) ) ) {
 			$notices[] = array( 'type' => $akismet_user->status );
 		}
@@ -1085,12 +1114,11 @@ class Akismet_Admin {
 		$notices[] = array( 'type' => 'new-key-invalid' );
 		$notices[] = array( 'type' => 'existing-key-invalid' );
 		$notices[] = array( 'type' => 'new-key-failed' );
-		$notices[] = array( 'type' => 'limit-reached', 'level' => 'yellow' );
-		$notices[] = array( 'type' => 'limit-reached', 'level' => 'red' );
-		$notices[] = array( 'type' => 'usage-limit', 'api_calls' => '15000', 'usage_limit' => '10000', 'upgrade_plan' => 'Enterprise', 'upgrade_url' => 'https://akismet.com/account/' );
+		$notices[] = array( 'type' => 'usage-limit', 'api_calls' => '15000', 'usage_limit' => '10000', 'upgrade_plan' => 'Enterprise', 'upgrade_url' => 'https://akismet.com/account/', 'code' => 10502 );
 		$notices[] = array( 'type' => 'spam-check-cron-disabled' );
+		$notices[] = array( 'type' => 'alert', 'code' => 123 );
 		*/
-		
+
 		Akismet::log( compact( 'stat_totals', 'akismet_user' ) );
 		Akismet::view( 'config', compact( 'api_key', 'akismet_user', 'stat_totals', 'notices' ) );
 	}
@@ -1120,18 +1148,18 @@ class Akismet_Admin {
 		elseif ( $hook_suffix == 'edit-comments.php' && wp_next_scheduled( 'akismet_schedule_cron_recheck' ) ) {
 			self::display_spam_check_warning();
 		}
-		
+
 		if ( isset( $_GET['akismet_recheck_complete'] ) ) {
 			$recheck_count = (int) $_GET['recheck_count'];
 			$spam_count = (int) $_GET['spam_count'];
-			
+
 			if ( $recheck_count === 0 ) {
 				$message = __( 'There were no comments to check. Akismet will only check comments awaiting moderation.', 'akismet' );
 			}
 			else {
 				$message = sprintf( _n( 'Akismet checked %s comment.', 'Akismet checked %s comments.', $recheck_count, 'akismet' ), number_format( $recheck_count ) );
 				$message .= ' ';
-			
+
 				if ( $spam_count === 0 ) {
 					$message .= __( 'No comments were caught as spam.', 'akismet' );
 				}
@@ -1139,7 +1167,7 @@ class Akismet_Admin {
 					$message .= sprintf( _n( '%s comment was caught as spam.', '%s comments were caught as spam.', $spam_count, 'akismet' ), number_format( $spam_count ) );
 				}
 			}
-			
+
 			echo '<div class="notice notice-success"><p>' . esc_html( $message ) . '</p></div>';
 		}
 		else if ( isset( $_GET['akismet_recheck_error'] ) ) {
@@ -1155,25 +1183,25 @@ class Akismet_Admin {
 			foreach ( self::$notices as $index => $type ) {
 				if ( is_object( $type ) ) {
 					$notice_header = $notice_text = '';
-					
+
 					if ( property_exists( $type, 'notice_header' ) ) {
 						$notice_header = wp_kses( $type->notice_header, self::$allowed );
 					}
-				
+
 					if ( property_exists( $type, 'notice_text' ) ) {
 						$notice_text = wp_kses( $type->notice_text, self::$allowed );
 					}
-					
+
 					if ( property_exists( $type, 'status' ) ) {
 						$type = wp_kses( $type->status, self::$allowed );
 						Akismet::view( 'notice', compact( 'type', 'notice_header', 'notice_text' ) );
-						
+
 						unset( self::$notices[ $index ] );
 					}
 				}
 				else {
 					Akismet::view( 'notice', compact( 'type' ) );
-					
+
 					unset( self::$notices[ $index ] );
 				}
 			}
@@ -1199,13 +1227,13 @@ class Akismet_Admin {
 
 		if ( !$xml->isError() ) {
 			$responses = $xml->getResponse();
-			if ( count( $responses ) > 1 ) {
+			if ( ( is_countable( $responses ) ? count( $responses ) : 0 ) > 1 ) {
 				// Due to a quirk in how Jetpack does multi-calls, the response order
 				// can't be trusted to match the call order. It's a good thing our
 				// return values can be mostly differentiated from each other.
 				$first_response_value = array_shift( $responses[0] );
 				$second_response_value = array_shift( $responses[1] );
-				
+
 				// If WPCOM ever reaches 100 billion users, this will fail. :-)
 				if ( preg_match( '/^[a-f0-9]{12}$/i', $first_response_value ) ) {
 					$api_key = $first_response_value;
@@ -1215,13 +1243,13 @@ class Akismet_Admin {
 					$api_key = $second_response_value;
 					$user_id = (int) $first_response_value;
 				}
-				
+
 				return compact( 'api_key', 'user_id' );
 			}
 		}
 		return false;
 	}
-	
+
 	/**
 	 * Some commentmeta isn't useful in an export file. Suppress it (when supported).
 	 *
@@ -1234,10 +1262,10 @@ class Akismet_Admin {
 		if ( in_array( $key, array( 'akismet_as_submitted', 'akismet_rechecking', 'akismet_delayed_moderation_email' ) ) ) {
 			return true;
 		}
-		
+
 		return $exclude;
 	}
-	
+
 	/**
 	 * When Akismet is active, remove the "Activate Akismet" step from the plugin description.
 	 */
@@ -1250,7 +1278,7 @@ class Akismet_Admin {
 				$all_plugins['akismet/akismet.php']['Description'] = __( 'Used by millions, Akismet is quite possibly the best way in the world to <strong>protect your blog from spam</strong>. It keeps your site protected even while you sleep. To get started, just go to <a href="admin.php?page=akismet-key-config">your Akismet Settings page</a> to set up your API key.', 'akismet' );
 			}
 		}
-		
+
 		return $all_plugins;
 	}
 
@@ -1259,7 +1287,7 @@ class Akismet_Admin {
 			update_option( 'akismet_comment_form_privacy_notice', $state );
 		}
 	}
-	
+
 	public static function register_personal_data_eraser( $erasers ) {
 		$erasers['akismet'] = array(
 			'eraser_friendly_name' => __( 'Akismet', 'akismet' ),
@@ -1268,14 +1296,14 @@ class Akismet_Admin {
 
 		return $erasers;
 	}
-	
+
 	/**
 	 * When a user requests that their personal data be removed, Akismet has a duty to discard
 	 * any personal data we store outside of the comment itself. Right now, that is limited
 	 * to the copy of the comment we store in the akismet_as_submitted commentmeta.
 	 *
 	 * FWIW, this information would be automatically deleted after 15 days.
-	 * 
+	 *
 	 * @param $email_address string The email address of the user who has requested erasure.
 	 * @param $page int This function can (and will) be called multiple times to prevent timeouts,
 	 *                  so this argument is used for pagination.
@@ -1284,7 +1312,7 @@ class Akismet_Admin {
 	 */
 	public static function erase_personal_data( $email_address, $page = 1 ) {
 		$items_removed = false;
-		
+
 		$number = 50;
 		$page = (int) $page;
 
@@ -1300,7 +1328,7 @@ class Akismet_Admin {
 
 		foreach ( (array) $comments as $comment ) {
 			$comment_as_submitted = get_comment_meta( $comment->comment_ID, 'akismet_as_submitted', true );
-			
+
 			if ( $comment_as_submitted ) {
 				delete_comment_meta( $comment->comment_ID, 'akismet_as_submitted' );
 				$items_removed = true;
@@ -1308,8 +1336,8 @@ class Akismet_Admin {
 		}
 
 		// Tell core if we have more comments to work on still
-		$done = count( $comments ) < $number;
-		
+		$done = ( is_countable( $comments ) ? count( $comments ) : 0 ) < $number;
+
 		return array(
 			'items_removed' => $items_removed,
 			'items_retained' => false, // always false in this example
@@ -1317,4 +1345,33 @@ class Akismet_Admin {
 			'done' => $done,
 		);
 	}
+
+	/**
+	 * Return an array of HTML elements that are allowed in a notice.
+	 *
+	 * @return array
+	 */
+	public static function get_notice_kses_allowed_elements() {
+		return self::$allowed;
+	}
+
+	/**
+	 * Return a version to append to the URL of an asset file (e.g. CSS and images).
+	 *
+	 * @param string $relative_path Relative path to asset file
+	 * @return string
+	 */
+	public static function get_asset_file_version( $relative_path ) {
+
+		$full_path = AKISMET__PLUGIN_DIR . $relative_path;
+
+		// If the AKISMET_VERSION contains a lower-case letter, it's a development version (e.g. 5.3.1a2).
+		// Use the file modified time in development.
+		if ( preg_match( '/[a-z]/', AKISMET_VERSION ) && file_exists( $full_path ) ) {
+			return filemtime( $full_path );
+		}
+
+		// Otherwise, use the AKISMET_VERSION.
+		return AKISMET_VERSION;
+	}
 }
